# -*- coding: utf-8 -*-
from threading import Thread
from modules import service_functions
from modules.settings_reader import make_settings_dict, get_setting
from modules.kodi_utils import set_property, clear_property, sleep, xbmc_monitor, logger

class BudsMonitor(xbmc_monitor):
	def __init__ (self):
		xbmc_monitor.__init__(self)
		logger('BUDS', 'Main Monitor Service Starting')
		logger('BUDS', 'Settings Monitor Service Starting')
		self.startUpServices()
	
	def startUpServices(self):
		threads = []
		functions = (service_functions.DatabaseMaintenance().run, service_functions.TraktMonitor().run)
		for item in functions: threads.append(Thread(target=item))
		while not self.abortRequested():
			try: service_functions.InitializeDatabases().run()
			except: pass
			try: service_functions.CheckSettingsFile().run()
			except: pass
			try: service_functions.SyncAllAccounts().run()
			except: pass
			[i.start() for i in threads]
			try: service_functions.ClearSubs().run()
			except: pass
			try: service_functions.ViewsSetWindowProperties().run()
			except: pass
			try: service_functions.AutoRun().run()
			except: pass
			try: service_functions.ReuseLanguageInvokerCheck().run()
			except: pass
			break

	def onScreensaverActivated(self):
		set_property('buds_pause_services', 'true')

	def onScreensaverDeactivated(self):
		clear_property('buds_pause_services')

	def onSettingsChanged(self):
		clear_property('buds_settings')
		sleep(50)
		make_settings_dict()
		set_property('buds_kodi_menu_cache', get_setting('kodi_menu_cache'))

	def onNotification(self, sender, method, data):
		if method == 'System.OnSleep': set_property('buds_pause_services', 'true')
		elif method == 'System.OnWake': clear_property('buds_pause_services')


BudsMonitor().waitForAbort()

logger('BUDS', 'Settings Monitor Service Finished')
logger('BUDS', 'Main Monitor Service Finished')